#!/usr/bin/env python

"""
A WebStack application for "playing" questionnaires designed by the
questionnaire editor.
"""

import WebStack.Generic
from XSLForms.Resources.WebResources import \
    XSLFormsResource, input, output, resources, prepare_resources as xslforms_prepare_resources
import os
import libxml2dom
from WebStack.Repositories.Directory import DirectoryRepository
import time

# Site map imports.

from WebStack.Resources.ResourceMap import SimpleMap
from WebStack.Resources.Selectors import EncodingSelector, PathSelector, AuthInfoSelector
from WebStack.Resources.Static import DirectoryResource
from WebStack.Resources.Login import LoginAuthenticator, LoginResource
from WebStack.Resources.LoginRedirect import LoginRedirectAuthenticator, SiteLoginRedirectResource

# Configuration settings.

encoding = "utf-8"
storage_directory = os.path.join(os.getcwd(), "Responses")

# Resource classes.

class QuestionnairePlayerResource(XSLFormsResource):

    "A resource providing a questionnaire editor."

    resource_dir = resources(__file__)
    template_resources = {
        "questionnaire" : output("player_template.xhtml")
        }
    init_resources = {
        "questionnaire" : input("player_template.xhtml")
        }
    document_resources = {
        "questions" : "questions.xml"
        }
    transform_resources = {
        "distill" : ["distill_responses.xsl"]
        }

    def select_activity(self, trans, form):
        form.set_activity("questionnaire")

    def create_document(self, trans, form):
        if XSLFormsResource.create_document(self, trans, form):
            admin = trans.get_user() == "admin"
            response = trans.get_virtual_path_info().split("/")

            repository = DirectoryRepository(storage_directory)
            if admin and len(response) > 1:
                key = response[1]
                try:
                    questions_xml = repository.lock(key)
                    try:
                        questions = libxml2dom.parse(questions_xml)
                    finally:
                        repository.unlock(key)
                except KeyError:
                    trans.set_response_code(404)
                    trans.set_content_type(WebStack.Generic.ContentType("text/plain"))
                    trans.get_response_stream().write("Questionnaire not found")
                    raise WebStack.Generic.EndOfResponse
            else:
                questions_xml = self.prepare_document("questions")
                questions = libxml2dom.parse(questions_xml)

            doc = form.get_document()
            imported_questions = doc.importNode(questions.documentElement, 1)
            doc.replaceChild(imported_questions, doc.documentElement)

    def respond_to_input(self, trans, form):

        parameters = form.get_parameters()
        selectors = form.get_selectors()
        questionnaire = form.get_document()

        # Send a response according to certain parameters.
        # When saved, an XML version of the data is employed.

        if parameters.has_key("finish"):
            repository = DirectoryRepository(storage_directory)
            key = str(time.time())
            output_filename = repository.lock(key, 1)
            distilled_output_filename = repository.lock("distilled-" + key, 1)
            try:
                distill_xsl = self.prepare_transform("distill")
                distilled = self.get_result(distill_xsl, questionnaire)
                distilled.toStream(open(distilled_output_filename, "wb"), "utf-8", prettyprint=1)
                questionnaire.toStream(open(output_filename, "wb"), "utf-8", prettyprint=1)
                questionnaire.documentElement.setAttribute("submitted", "true")
            finally:
                repository.unlock(key)
                repository.unlock("distilled-" + key)

    def create_output(self, trans, form):
        XSLFormsResource.create_output(self, trans, form, stylesheet_parameters=trans.get_attributes())

class QuestionnaireAdminResource(XSLFormsResource):

    "A resource providing a list of questionnaires."

    resource_dir = resources(__file__)
    template_resources = {
        "questionnaires" : output("admin_template.xhtml")
        }
    init_resources = {
        "questionnaires" : input("admin_template.xhtml")
        }

    def select_activity(self, trans, form):
        form.set_activity("questionnaires")

    def create_document(self, trans, form):
        if XSLFormsResource.create_document(self, trans, form):
            doc = form.get_document()
            questionnaires = doc.documentElement
            for filename in os.listdir(storage_directory):
                if filename[0].isdigit():
                    questionnaire = doc.createElement("questionnaire")
                    questionnaire.setAttribute("name", filename)
                    questionnaire.setAttribute("date-text", time.strftime("%Y-%m-%d %H:%M:%S", time.gmtime(float(filename))))
                    questionnaires.appendChild(questionnaire)

    def create_output(self, trans, form):
        XSLFormsResource.create_output(self, trans, form, stylesheet_parameters=trans.get_attributes())

# Site map initialisation.

def get_site():

    "Return a simple Web site resource."

    # Authentication objects.

    secret_key="this_must_be_secret"
    credentials = [
        ("admin", "123"),
        ]
    login_authenticator = LoginAuthenticator(secret_key, credentials)
    login_redirect_authenticator = LoginRedirectAuthenticator(secret_key)

    # Get the main resource and the directory used by the application.

    questionnaire_resource = QuestionnairePlayerResource()
    admin_resource = QuestionnaireAdminResource()
    directory = questionnaire_resource.resource_dir

    # Get some login resources.

    login_redirector = SiteLoginRedirectResource(
        authenticator=login_redirect_authenticator,
        login_url="login",
        app_url="",
        resource=SimpleMap({
            "" : admin_resource,
            None : questionnaire_resource
            })
        )
    login_redirector.logout_parameter_name="logout"

    login_resource = LoginResource(login_authenticator)

    # Make a simple Web site.

    resource = SimpleMap({
        # Static resources:
        "styles" : DirectoryResource(os.path.join(directory, "styles"), {"css" : "text/css"}),
        # Login resource:
        "login" : login_resource,
        # Administration resource:
        "admin" : login_redirector,
        # Main page:
        "" : questionnaire_resource
        })

    return EncodingSelector(PathSelector(AuthInfoSelector(resource, login_redirect_authenticator)), encoding)

# Resource preparation ahead of time - useful for making installations.

def prepare_resources():
    for cls in [QuestionnairePlayerResource, QuestionnaireAdminResource]:
        xslforms_prepare_resources(cls)

# vim: tabstop=4 expandtab shiftwidth=4
